@@ -142,6 +142,7 @@ class MainWindow(QMainWindow):
         self.ConfigDlg = None
         self.plotSettingDlg = None
         self.custom_plot_settings = {}
+        self._last_export_dir = '.'
 
         self.setAttribute(Qt.WA_DeleteOnClose)
         if os.name == 'nt':
@@ -211,6 +212,11 @@ class MainWindow(QMainWindow):
                                  statusTip="Export commands to script",
                                  triggered=self.saveScript)
 
+        self.clipboardAct = QAction("Copy image to clipboard", self,
+                                    statusTip="Copy current output to clipboard as png",
+                                    triggered=self.export_clipboard)
+        self.clipboardAct.setEnabled(False)
+
         self.reLoadAct = QAction("Re-load files", self,
                                  statusTip="Re-scan directory",
                                  triggered=self.reLoad)
@@ -287,6 +293,7 @@ class MainWindow(QMainWindow):
         self.fileMenu = menu.addMenu("&File")
         self.fileMenu.addAction(self.openChainsAct)
         self.fileMenu.addAction(self.exportAct)
+        self.fileMenu.addAction(self.clipboardAct)
         self.fileMenu.addAction(self.scriptAct)
         self.separatorAct = self.fileMenu.addSeparator()
         self.fileMenu.addAction(self.reLoadAct)
@@ -441,6 +448,16 @@ class MainWindow(QMainWindow):
         self.comboBoxColor.clear()
         self.comboBoxColor.setEnabled(False)
 
+        self.toggleZ = QRadioButton("Z-axis:", self.selectWidget)
+        self.connect(self.toggleZ, SIGNAL("toggled(bool)"), self.statusPlotType)
+        self.comboBoxZ = QComboBox(self)
+        self.comboBoxZ.clear()
+        self.comboBoxZ.setEnabled(False)
+
+        self.checkShadow = QCheckBox("Shadows", self.selectWidget)
+        self.checkShadow.setCheckState(Qt.Unchecked)
+        self.checkShadow.setVisible(False)
+
         self.toggleFilled.setChecked(True)
 
         self.trianglePlot = QCheckBox("Triangle Plot", self.selectWidget)
@@ -472,17 +489,24 @@ class MainWindow(QMainWindow):
 
         leftLayout.addWidget(self.selectAllX, 5, 0, 1, 2)
         leftLayout.addWidget(self.selectAllY, 5, 2, 1, 2)
-        leftLayout.addWidget(self.listParametersX, 6, 0, 5, 2)
+        leftLayout.addWidget(self.listParametersX, 6, 0, 6, 2)
         leftLayout.addWidget(self.listParametersY, 6, 2, 1, 2)
+
         leftLayout.addWidget(self.toggleFilled, 7, 2, 1, 1)
         leftLayout.addWidget(self.checkInsideLegend, 7, 3, 1, 1)
         leftLayout.addWidget(self.toggleLine, 8, 2, 1, 1)
         leftLayout.addWidget(self.checkShade, 8, 3, 1, 1)
-        leftLayout.addWidget(self.toggleColor, 9, 2, 1, 1)
-        leftLayout.addWidget(self.comboBoxColor, 9, 3, 1, 1)
-        leftLayout.addWidget(self.trianglePlot, 10, 2, 1, 2)
 
-        leftLayout.addWidget(self.pushButtonPlot, 12, 0, 1, 4)
+        leftLayout.addWidget(self.toggleZ, 9, 2, 1, 1)
+        leftLayout.addWidget(self.comboBoxZ, 9, 3, 1, 1)
+
+        leftLayout.addWidget(self.toggleColor, 10, 2, 1, 1)
+        leftLayout.addWidget(self.comboBoxColor, 10, 3, 1, 1)
+
+        leftLayout.addWidget(self.trianglePlot, 11, 2, 1, 1)
+        leftLayout.addWidget(self.checkShadow, 11, 3, 1, 1)
+
+        leftLayout.addWidget(self.pushButtonPlot, 13, 0, 1, 4)
 
         self.selectWidget.setLayout(leftLayout)
 
@@ -661,15 +685,24 @@ class MainWindow(QMainWindow):
             plotter = self.plotter_script
 
         if plotter:
-            filename, _ = QFileDialog.getSaveFileName(
-                self, "Choose a file name", '.', "PDF (*.pdf);; Image (*.png *.jpg)")
+            filename, _ = QFileDialog.getSaveFileName(self, "Choose a file name", self._last_export_dir,
+                                                      "PDF (*.pdf);; Image (*.png *.jpg)")
             if not filename:
                 return
+            self._last_export_dir = os.path.dirname(filename)
             filename = str(filename)
             plotter.export(filename)
+
         else:
             self.warning("Export", "No plotter data to export")
 
+    def export_clipboard(self):
+        """
+        Callback for action clipboard copy image'.
+        """
+        index = self.tabWidget.currentIndex()
+        self.updateScriptPreview(self.plotter if index == 0 else self.plotter_script, True)
+
     def saveScript(self):
         """
         Callback for action 'Save script'.
@@ -685,13 +718,15 @@ class MainWindow(QMainWindow):
             self.warning("Script", "No script to save")
             return
 
-        filename, _ = QFileDialog.getSaveFileName(self, "Choose a file name", '.', "Python (*.py)")
+        filename, _ = QFileDialog.getSaveFileName(self, "Choose a file name",
+                                                  self._last_export_dir, "Python (*.py)")
         if not filename:
             return
         filename = str(filename)
-        logging.debug("Export script to %s" % filename)
+        logging.debug("Export script to %s", filename)
         with open(filename, 'w', encoding="utf-8") as f:
             f.write(script)
+        self._last_export_dir = os.path.dirname(filename)
 
     def reLoad(self):
         adir = self.getSettings().value('lastSearchDirectory')
@@ -1193,7 +1228,8 @@ class MainWindow(QMainWindow):
         self._updateListParametersSelection(old_selection["x"], self.listParametersX)
         self._updateListParametersSelection(old_selection["y"], self.listParametersY)
 
-        self._updateComboBoxColor(self.paramNames.list())
+        self._updateComboBoxParam(self.comboBoxColor, self.paramNames.list())
+        self._updateComboBoxParam(self.comboBoxZ, list(self.paramNamesTags))
 
     def _resetPlotData(self):
         # Script
@@ -1416,6 +1452,9 @@ class MainWindow(QMainWindow):
         return [self.paramNamesTags[tag]
                 for tag in self.getCheckedParams(self.listParametersY)]
 
+    def getZParam(self):
+        return self.paramNamesTags[str(self.comboBoxZ.currentText())]
+
     def statusSelectAllX(self):
         """
         Slot function called when selectAllX is modified.
@@ -1441,7 +1480,11 @@ class MainWindow(QMainWindow):
     def statusPlotType(self, checked):
         # radio buttons changed
         self.checkShade.setEnabled(self.toggleLine.isChecked())
-        self.comboBoxColor.setEnabled(self.toggleColor.isChecked())
+        self.comboBoxColor.setEnabled(self.toggleColor.isChecked() or self.toggleZ.isChecked())
+        self.comboBoxZ.setEnabled(self.toggleZ.isChecked())
+        if self.toggleZ.isChecked():
+            self.trianglePlot.setCheckState(Qt.Unchecked)
+        self.checkShadow.setVisible(self.toggleZ.isChecked())
 
     def statusTriangle(self, checked):
         self.checkInsideLegend.setVisible(
@@ -1454,16 +1497,16 @@ class MainWindow(QMainWindow):
             self.trianglePlot.checkState() != Qt.Checked)
         self.checkInsideLegend.setEnabled(self.checkInsideLegend.isVisible())
 
-    def _updateComboBoxColor(self, listOfParams):
+    def _updateComboBoxParam(self, combo, listOfParams):
         if self.rootdirname and os.path.isdir(self.rootdirname):
-            param_old = str(self.comboBoxColor.currentText())
+            param_old = str(combo.currentText())
             param_old_new_name = getattr(
                 self.paramNames.parWithName(param_old), "name", None)
-            self.comboBoxColor.clear()
-            self.comboBoxColor.addItems(listOfParams)
-            idx = self.comboBoxColor.findText(param_old_new_name, Qt.MatchExactly)
+            combo.clear()
+            combo.addItems(listOfParams)
+            idx = combo.findText(param_old_new_name, Qt.MatchExactly)
             if idx != -1:
-                self.comboBoxColor.setCurrentIndex(idx)
+                combo.setCurrentIndex(idx)
 
     def checkedRootNames(self):
         items = []
@@ -1641,7 +1684,26 @@ class MainWindow(QMainWindow):
                     script += ")\n"
                 else:
                     raise GuiSelectionError("Select more than 1 x parameter for triangle plot")
-
+            elif self.toggleZ.isChecked():
+                z_param = self.getZParam()
+                if len(items_x) == 1 and len(items_y) == 1 and z_param:
+                    params = [items_x[0], items_y[0], z_param]
+                    if color_param:
+                        params.append(color_param)
+                    logging.debug("4d plot with params = %s" % str(params))
+                    script += "params = %s\n" % str(params)
+                    setSizeForN(1, 1)
+                    cols = [c[-1] for c in self.plotter.settings.line_styles[:len(roots) - 1]]
+                    self.plotter.plot_4d(roots, params, color_bar=z_param, compare_colors=cols,
+                                         shadow_color=self.checkShadow.isChecked())
+                    script += "g.plot_4d(roots, params, color_bar=True%s%s)\n" % ("" if len(roots) == 1 else
+                                                                                  ", compare_colors=%r" % cols,
+                                                                                  ", shadow_color=True" if
+                                                                                  self.checkShadow.isChecked() else "")
+                    self.updatePlot()
+                else:
+                    raise GuiSelectionError("For an x-y-z plot select one parameter of each, and optionally a "
+                                            "parameter to color by")
             elif len(items_x) > 0 and len(items_y) == 0:
                 # 1D plot
                 actionText = "1D plot"
@@ -1742,6 +1804,7 @@ class MainWindow(QMainWindow):
             script += "g.export()\n"
             self.script = script
             self.exportAct.setEnabled(True)
+            self.clipboardAct.setEnabled(True)
         except Exception as e:
             QApplication.restoreOverrideCursor()
             self.errorReport(e, caption=actionText)
@@ -1801,7 +1864,7 @@ class MainWindow(QMainWindow):
 
     def openScript(self):
         filename, _ = QFileDialog.getOpenFileName(
-            self, "Choose a file name", '.', "Python (*.py)")
+            self, "Choose a file name", self._last_export_dir, "Python (*.py)", )
         if not filename:
             return
         filename = str(filename)
@@ -1809,6 +1872,7 @@ class MainWindow(QMainWindow):
         with open(filename, 'r', encoding='utf-8-sig') as f:
             self.script_edit = f.read()
         self.textWidget.setPlainText(self.script_edit)
+        self._last_export_dir = os.path.dirname(filename)
 
     def clearScript(self):
         self.textWidget.clear()
@@ -1853,6 +1917,7 @@ class MainWindow(QMainWindow):
                     break
 
             self.exportAct.setEnabled(True)
+            self.clipboardAct.setEnabled(True)
 
         except SyntaxError as e:
             self.warning("Plot script", type(e).__name__ + ': %s\n %s' % (e, e.text))
@@ -1865,28 +1930,29 @@ class MainWindow(QMainWindow):
             self._set_rc(oldrc)
             self.showMessage()
 
-    def updateScriptPreview(self, plotter):
+    def updateScriptPreview(self, plotter, clipboard=False):
         if plotter.fig is None:
             return
 
-        self.plotter_script = plotter
-
-        i = 0
-        while True:
-            item = self.plotWidget2.layout().takeAt(i)
-            if item is None:
-                break
-            del item
+        if not clipboard:
+            self.plotter_script = plotter
+            while True:
+                item = self.plotWidget2.layout().takeAt(0)
+                if item is None:
+                    break
+                del item
 
         # Save in PNG format, and display it in a QLabel
         buf = BytesIO()
-
+        dpi = self.logicalDpiX() * self.devicePixelRatio()
+        if clipboard:
+            dpi = max(100, dpi)
         plotter.fig.savefig(
             buf,
             format='png',
             edgecolor='w',
             facecolor='w',
-            dpi=self.logicalDpiX() * self.devicePixelRatio(),
+            dpi=dpi,
             bbox_extra_artists=plotter.extra_artists,
             bbox_inches='tight')
         buf.seek(0)
@@ -1894,18 +1960,21 @@ class MainWindow(QMainWindow):
         # noinspection PyTypeChecker
         image = QImage.fromData(buf.getvalue())
 
-        pixmap = QPixmap.fromImage(image)
-        pixmap.setDevicePixelRatio(self.devicePixelRatio())
-        label = QLabel(self.scrollArea)
-        label.setPixmap(pixmap)
+        if clipboard:
+            QApplication.clipboard().setImage(image)
+        else:
+            pixmap = QPixmap.fromImage(image)
+            pixmap.setDevicePixelRatio(self.devicePixelRatio())
+            label = QLabel(self.scrollArea)
+            label.setPixmap(pixmap)
 
-        self.scrollArea = QScrollArea(self.plotWidget2)
-        self.scrollArea.setWidget(label)
-        self.scrollArea.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
+            self.scrollArea = QScrollArea(self.plotWidget2)
+            self.scrollArea.setWidget(label)
+            self.scrollArea.setAlignment(Qt.AlignHCenter | Qt.AlignVCenter)
 
-        self.plotWidget2.layout().addWidget(self.scrollArea)
-        self.plotWidget2.layout()
-        self.plotWidget2.show()
+            self.plotWidget2.layout().addWidget(self.scrollArea)
+            self.plotWidget2.layout()
+            self.plotWidget2.show()
 
 
 # ==============================================================================
@@ -2113,9 +2182,11 @@ class DialogParamTables(DialogTextOutput):
         clipboard.setText(self.tables[self.tabWidget.currentIndex()].tableTex())
 
     def saveLatex(self):
-        filename, _ = QFileDialog.getSaveFileName(self, "Choose a file name", '.', "Latex (*.tex)")
+        filename, _ = QFileDialog.getSaveFileName(
+            self, "Choose a file name", self._last_export_dir, "Latex (*.tex)")
         if not filename:
             return
+        self._last_export_dir = os.path.dirname(filename)
         self.tables[self.tabWidget.currentIndex()].write(str(filename))
 
 
